@@ -30,7 +30,7 @@ class NeuralNetwork(nn.Module):
         super(NeuralNetwork, self).__init__()
 
     def get_device(self):
-        if next(self.parameters()).device == "cpu":
+        if next(self.parameters()).device.type == "cpu":
             return "cpu"
         else:
             return next(self.parameters()).device.index
@@ -56,7 +56,7 @@ class SegmentationNetwork(NeuralNetwork):
         # we need to know this because we need to know if we are a 2d or a 3d netowrk
         self.conv_op = None  # nn.Conv2d or nn.Conv3d
 
-        # this tells us how many channely we have in the output. Important for preallocation in inference
+        # this tells us how many channels we have in the output. Important for preallocation in inference
         self.num_classes = None  # number of channels in the output
 
         # depending on the loss, we do not hard code a nonlinearity into the architecture. To aggregate predictions
@@ -108,13 +108,13 @@ class SegmentationNetwork(NeuralNetwork):
         :param mixed_precision: if True, will run inference in mixed precision with autocast()
         :return:
         """
+        torch.cuda.empty_cache()
+
         assert step_size <= 1, 'step_size must be smaller than 1. Otherwise there will be a gap between consecutive ' \
                                'predictions'
 
         if verbose: print("debug: mirroring", do_mirroring, "mirror_axes", mirror_axes)
 
-        assert self.get_device() != "cpu", "CPU not implemented"
-
         if pad_kwargs is None:
             pad_kwargs = {'constant_values': 0}
 
@@ -198,6 +198,8 @@ class SegmentationNetwork(NeuralNetwork):
         :param verbose: Do you want a wall of text? If yes then set this to True
         :return:
         """
+        torch.cuda.empty_cache()
+
         assert step_size <= 1, 'step_size must be smaler than 1. Otherwise there will be a gap between consecutive ' \
                                'predictions'
 
@@ -206,8 +208,6 @@ class SegmentationNetwork(NeuralNetwork):
 
         if verbose: print("debug: mirroring", do_mirroring, "mirror_axes", mirror_axes)
 
-        assert self.get_device() != "cpu", "CPU not implemented"
-
         if pad_kwargs is None:
             pad_kwargs = {'constant_values': 0}
 
@@ -264,7 +264,7 @@ class SegmentationNetwork(NeuralNetwork):
         assert 0 < step_size <= 1, 'step_size must be larger than 0 and smaller or equal to 1'
 
         # our step width is patch_size*step_size at most, but can be narrower. For example if we have image size of
-        # 110, patch size of 32 and step_size of 0.5, then we want to make 4 steps starting at coordinate 0, 27, 55, 78
+        # 110, patch size of 64 and step_size of 0.5, then we want to make 3 steps starting at coordinate 0, 23, 46
         target_step_sizes_in_voxels = [i * step_size for i in patch_size]
 
         num_steps = [int(np.ceil((i - k) / j)) + 1 for i, j, k in zip(image_size, target_step_sizes_in_voxels, patch_size)]
@@ -290,12 +290,10 @@ class SegmentationNetwork(NeuralNetwork):
                                           verbose: bool) -> Tuple[np.ndarray, np.ndarray]:
         # better safe than sorry
         assert len(x.shape) == 4, "x must be (c, x, y, z)"
-        assert self.get_device() != "cpu"
+
         if verbose: print("step_size:", step_size)
         if verbose: print("do mirror:", do_mirroring)
 
-        torch.cuda.empty_cache()
-
         assert patch_size is not None, "patch_size cannot be None for tiled prediction"
 
         # for sliding window inference the image must at least be as large as the patch size. It does not matter
@@ -327,8 +325,11 @@ class SegmentationNetwork(NeuralNetwork):
                 if verbose: print("using precomputed Gaussian")
                 gaussian_importance_map = self._gaussian_3d
 
-            gaussian_importance_map = torch.from_numpy(gaussian_importance_map).cuda(self.get_device(),
-                                                                                     non_blocking=True)
+            gaussian_importance_map = torch.from_numpy(gaussian_importance_map)
+
+            #predict on cpu if cuda not available
+            if torch.cuda.is_available():
+                gaussian_importance_map = gaussian_importance_map.cuda(self.get_device(), non_blocking=True)
 
         else:
             gaussian_importance_map = None
@@ -356,10 +357,11 @@ class SegmentationNetwork(NeuralNetwork):
 
             if verbose: print("moving data to GPU")
             data = torch.from_numpy(data).cuda(self.get_device(), non_blocking=True)
-
+            
             if verbose: print("initializing result_numsamples (on GPU)")
             aggregated_nb_of_predictions = torch.zeros([self.num_classes] + list(data.shape[1:]), dtype=torch.half,
                                                        device=self.get_device())
+            
         else:
             if use_gaussian and num_tiles > 1:
                 add_for_nb_of_preds = self._gaussian_3d
@@ -430,13 +432,11 @@ class SegmentationNetwork(NeuralNetwork):
         This one does fully convolutional inference. No sliding window
         """
         assert len(x.shape) == 3, "x must be (c, x, y)"
-        assert self.get_device() != "cpu"
+
         assert self.input_shape_must_be_divisible_by is not None, 'input_shape_must_be_divisible_by must be set to ' \
                                                                   'run _internal_predict_2D_2Dconv'
         if verbose: print("do mirror:", do_mirroring)
 
-        torch.cuda.empty_cache()
-
         data, slicer = pad_nd_image(x, min_size, pad_border_mode, pad_kwargs, True,
                                     self.input_shape_must_be_divisible_by)
 
@@ -468,13 +468,11 @@ class SegmentationNetwork(NeuralNetwork):
         This one does fully convolutional inference. No sliding window
         """
         assert len(x.shape) == 4, "x must be (c, x, y, z)"
-        assert self.get_device() != "cpu"
+
         assert self.input_shape_must_be_divisible_by is not None, 'input_shape_must_be_divisible_by must be set to ' \
                                                                   'run _internal_predict_3D_3Dconv'
         if verbose: print("do mirror:", do_mirroring)
 
-        torch.cuda.empty_cache()
-
         data, slicer = pad_nd_image(x, min_size, pad_border_mode, pad_kwargs, True,
                                     self.input_shape_must_be_divisible_by)
 
@@ -502,15 +500,23 @@ class SegmentationNetwork(NeuralNetwork):
                                            do_mirroring: bool = True,
                                            mult: np.ndarray or torch.tensor = None) -> torch.tensor:
         assert len(x.shape) == 5, 'x must be (b, c, x, y, z)'
-        # everything in here takes place on the GPU. If x and mult are not yet on GPU this will be taken care of here
-        # we now return a cuda tensor! Not numpy array!
 
-        x = to_cuda(maybe_to_torch(x), gpu_id=self.get_device())
+        # if cuda available:
+        #   everything in here takes place on the GPU. If x and mult are not yet on GPU this will be taken care of here
+        #   we now return a cuda tensor! Not numpy array!
+
+        x = maybe_to_torch(x)
         result_torch = torch.zeros([1, self.num_classes] + list(x.shape[2:]),
-                                   dtype=torch.float).cuda(self.get_device(), non_blocking=True)
+                                   dtype=torch.float)
+
+        if torch.cuda.is_available():
+            x = to_cuda(x, gpu_id=self.get_device())
+            result_torch = result_torch.cuda(self.get_device(), non_blocking=True)
 
         if mult is not None:
-            mult = to_cuda(maybe_to_torch(mult), gpu_id=self.get_device())
+            mult = maybe_to_torch(mult)
+            if torch.cuda.is_available():
+                mult = to_cuda(mult, gpu_id=self.get_device())
 
         if do_mirroring:
             mirror_idx = 8
@@ -560,16 +566,23 @@ class SegmentationNetwork(NeuralNetwork):
     def _internal_maybe_mirror_and_pred_2D(self, x: Union[np.ndarray, torch.tensor], mirror_axes: tuple,
                                            do_mirroring: bool = True,
                                            mult: np.ndarray or torch.tensor = None) -> torch.tensor:
-        # everything in here takes place on the GPU. If x and mult are not yet on GPU this will be taken care of here
-        # we now return a cuda tensor! Not numpy array!
+        # if cuda available:
+        #   everything in here takes place on the GPU. If x and mult are not yet on GPU this will be taken care of here
+        #   we now return a cuda tensor! Not numpy array!
+        
         assert len(x.shape) == 4, 'x must be (b, c, x, y)'
 
-        x = to_cuda(maybe_to_torch(x), gpu_id=self.get_device())
-        result_torch = torch.zeros([x.shape[0], self.num_classes] + list(x.shape[2:]),
-                                   dtype=torch.float).cuda(self.get_device(), non_blocking=True)
+        x = maybe_to_torch(x)
+        result_torch = torch.zeros([x.shape[0], self.num_classes] + list(x.shape[2:]), dtype=torch.float)
+
+        if torch.cuda.is_available():
+            x = to_cuda(x, gpu_id=self.get_device())
+            result_torch = result_torch.cuda(self.get_device(), non_blocking=True)
 
         if mult is not None:
-            mult = to_cuda(maybe_to_torch(mult), gpu_id=self.get_device())
+            mult = maybe_to_torch(mult)
+            if torch.cuda.is_available():
+                mult = to_cuda(mult, gpu_id=self.get_device())
 
         if do_mirroring:
             mirror_idx = 4
@@ -606,12 +619,10 @@ class SegmentationNetwork(NeuralNetwork):
                                           verbose: bool) -> Tuple[np.ndarray, np.ndarray]:
         # better safe than sorry
         assert len(x.shape) == 3, "x must be (c, x, y)"
-        assert self.get_device() != "cpu"
+
         if verbose: print("step_size:", step_size)
         if verbose: print("do mirror:", do_mirroring)
 
-        torch.cuda.empty_cache()
-
         assert patch_size is not None, "patch_size cannot be None for tiled prediction"
 
         # for sliding window inference the image must at least be as large as the patch size. It does not matter
@@ -643,8 +654,10 @@ class SegmentationNetwork(NeuralNetwork):
                 if verbose: print("using precomputed Gaussian")
                 gaussian_importance_map = self._gaussian_2d
 
-            gaussian_importance_map = torch.from_numpy(gaussian_importance_map).cuda(self.get_device(),
-                                                                                     non_blocking=True)
+            gaussian_importance_map = torch.from_numpy(gaussian_importance_map)
+            if torch.cuda.is_available():
+                gaussian_importance_map = gaussian_importance_map.cuda(self.get_device(), non_blocking=True)
+
         else:
             gaussian_importance_map = None
 
