@@ -13,6 +13,8 @@
 #    limitations under the License.
 
 from collections import OrderedDict
+from copy import deepcopy
+
 from batchgenerators.augmentations.utils import resize_segmentation
 from nnunet.configuration import default_num_threads, RESAMPLING_SEPARATE_Z_ANISO_THRESHOLD
 from nnunet.preprocessing.cropping import get_case_identifier_from_npz, ImageCropper
@@ -34,11 +36,9 @@ def get_lowres_axis(new_spacing):
 
 
 def resample_patient(data, seg, original_spacing, target_spacing, order_data=3, order_seg=0, force_separate_z=False,
-                     cval_data=0, cval_seg=-1, order_z_data=0, order_z_seg=0,
+                     order_z_data=0, order_z_seg=0,
                      separate_z_anisotropy_threshold=RESAMPLING_SEPARATE_Z_ANISO_THRESHOLD):
     """
-    :param cval_seg:
-    :param cval_data:
     :param data:
     :param seg:
     :param original_spacing:
@@ -85,28 +85,28 @@ def resample_patient(data, seg, original_spacing, target_spacing, order_data=3,
 
     if axis is not None:
         if len(axis) == 3:
-            # every axis has the spacing
-            axis = (0, )
+            # every axis has the spacing, this should never happen, why is this code here?
+            do_separate_z = False
         elif len(axis) == 2:
-            print("WARNING: axis has len 2, axis: %s, spacing: %s, target_spacing: %s" % (str(axis), original_spacing, target_spacing))
+            # this happens for spacings like (0.24, 1.25, 1.25) for example. In that case we do not want to resample
+            # separately in the out of plane axis
             do_separate_z = False
         else:
             pass
 
     if data is not None:
-        data_reshaped = resample_data_or_seg(data, new_shape, False, axis, order_data, do_separate_z, cval=cval_data,
+        data_reshaped = resample_data_or_seg(data, new_shape, False, axis, order_data, do_separate_z,
                                              order_z=order_z_data)
     else:
         data_reshaped = None
     if seg is not None:
-        seg_reshaped = resample_data_or_seg(seg, new_shape, True, axis, order_seg, do_separate_z, cval=cval_seg,
-                                            order_z=order_z_seg)
+        seg_reshaped = resample_data_or_seg(seg, new_shape, True, axis, order_seg, do_separate_z, order_z=order_z_seg)
     else:
         seg_reshaped = None
     return data_reshaped, seg_reshaped
 
 
-def resample_data_or_seg(data, new_shape, is_seg, axis=None, order=3, do_separate_z=False, cval=0, order_z=0):
+def resample_data_or_seg(data, new_shape, is_seg, axis=None, order=3, do_separate_z=False, order_z=0):
     """
     separate_z=True will resample with order 0 along z
     :param data:
@@ -127,10 +127,10 @@ def resample_data_or_seg(data, new_shape, is_seg, axis=None, order=3, do_separat
         resize_fn = resize
         kwargs = {'mode': 'edge', 'anti_aliasing': False}
     dtype_data = data.dtype
-    data = data.astype(float)
     shape = np.array(data[0].shape)
     new_shape = np.array(new_shape)
     if np.any(shape != new_shape):
+        data = data.astype(float)
         if do_separate_z:
             print("separate z, order in z is", order_z, "order inplane is", order)
             assert len(axis) == 1, "only one anisotropic axis supported"
@@ -147,11 +147,11 @@ def resample_data_or_seg(data, new_shape, is_seg, axis=None, order=3, do_separat
                 reshaped_data = []
                 for slice_id in range(shape[axis]):
                     if axis == 0:
-                        reshaped_data.append(resize_fn(data[c, slice_id], new_shape_2d, order, cval=cval, **kwargs))
+                        reshaped_data.append(resize_fn(data[c, slice_id], new_shape_2d, order, **kwargs))
                     elif axis == 1:
-                        reshaped_data.append(resize_fn(data[c, :, slice_id], new_shape_2d, order, cval=cval, **kwargs))
+                        reshaped_data.append(resize_fn(data[c, :, slice_id], new_shape_2d, order, **kwargs))
                     else:
-                        reshaped_data.append(resize_fn(data[c, :, :, slice_id], new_shape_2d, order, cval=cval,
+                        reshaped_data.append(resize_fn(data[c, :, :, slice_id], new_shape_2d, order,
                                                        **kwargs))
                 reshaped_data = np.stack(reshaped_data, axis)
                 if shape[axis] != new_shape[axis]:
@@ -171,7 +171,7 @@ def resample_data_or_seg(data, new_shape, is_seg, axis=None, order=3, do_separat
 
                     coord_map = np.array([map_rows, map_cols, map_dims])
                     if not is_seg or order_z == 0:
-                        reshaped_final_data.append(map_coordinates(reshaped_data, coord_map, order=order_z, cval=cval,
+                        reshaped_final_data.append(map_coordinates(reshaped_data, coord_map, order=order_z,
                                                                    mode='nearest')[None])
                     else:
                         unique_labels = np.unique(reshaped_data)
@@ -180,7 +180,7 @@ def resample_data_or_seg(data, new_shape, is_seg, axis=None, order=3, do_separat
                         for i, cl in enumerate(unique_labels):
                             reshaped_multihot = np.round(
                                 map_coordinates((reshaped_data == cl).astype(float), coord_map, order=order_z,
-                                                cval=cval, mode='nearest'))
+                                                mode='nearest'))
                             reshaped[reshaped_multihot > 0.5] = cl
                         reshaped_final_data.append(reshaped[None])
                 else:
@@ -190,7 +190,7 @@ def resample_data_or_seg(data, new_shape, is_seg, axis=None, order=3, do_separat
             print("no separate z, order", order)
             reshaped = []
             for c in range(data.shape[0]):
-                reshaped.append(resize_fn(data[c], new_shape, order, cval=cval, **kwargs)[None])
+                reshaped.append(resize_fn(data[c], new_shape, order, **kwargs)[None])
             reshaped_final_data = np.vstack(reshaped)
         return reshaped_final_data.astype(dtype_data)
     else:
@@ -293,13 +293,18 @@ class GenericPreprocessor(object):
                 data[c] = (data[c] - mn) / sd
                 if use_nonzero_mask[c]:
                     data[c][seg[-1] < 0] = 0
+            elif scheme == 'noNorm':
+                pass
             else:
                 if use_nonzero_mask[c]:
                     mask = seg[-1] >= 0
+                    data[c][mask] = (data[c][mask] - data[c][mask].mean()) / (data[c][mask].std() + 1e-8)
+                    data[c][mask == 0] = 0
                 else:
-                    mask = np.ones(seg.shape[1:], dtype=bool)
-                data[c][mask] = (data[c][mask] - data[c][mask].mean()) / (data[c][mask].std() + 1e-8)
-                data[c][mask == 0] = 0
+                    mn = data[c].mean()
+                    std = data[c].std()
+                    # print(data[c].shape, data[c].dtype, mn, std)
+                    data[c] = (data[c] - mn) / (std + 1e-8)
         return data, seg, properties
 
     def preprocess_test_case(self, data_files, target_spacing, seg_file=None, force_separate_z=None):
@@ -337,7 +342,8 @@ class GenericPreprocessor(object):
                 class_locs[c] = []
                 continue
             target_num_samples = min(num_samples, len(all_locs))
-            target_num_samples = max(target_num_samples, len(all_locs) * min_percent_coverage)
+            target_num_samples = max(target_num_samples, int(np.ceil(len(all_locs) * min_percent_coverage)))
+
             selected = all_locs[rndst.choice(len(all_locs), target_num_samples, replace=False)]
             class_locs[c] = selected
             print(c, target_num_samples)
@@ -462,6 +468,8 @@ class Preprocessor3DDifferentResampling(GenericPreprocessor):
                 data[c] = (data[c] - mn) / sd
                 if use_nonzero_mask[c]:
                     data[c][seg[-1] < 0] = 0
+            elif scheme == 'noNorm':
+                pass
             else:
                 if use_nonzero_mask[c]:
                     mask = seg[-1] >= 0
@@ -556,6 +564,8 @@ class Preprocessor3DBetterResampling(GenericPreprocessor):
                 data[c] = (data[c] - mn) / sd
                 if use_nonzero_mask[c]:
                     data[c][seg[-1] < 0] = 0
+            elif scheme == 'noNorm':
+                pass
             else:
                 if use_nonzero_mask[c]:
                     mask = seg[-1] >= 0
@@ -656,6 +666,8 @@ class PreprocessorFor2D(GenericPreprocessor):
                 data[c] = (data[c] - mn) / sd
                 if use_nonzero_mask[c]:
                     data[c][seg[-1] < 0] = 0
+            elif scheme == 'noNorm':
+                pass
             else:
                 if use_nonzero_mask[c]:
                     mask = seg[-1] >= 0
@@ -667,6 +679,182 @@ class PreprocessorFor2D(GenericPreprocessor):
         return data, seg, properties
 
 
+class PreprocessorFor3D_LeaveOriginalZSpacing(GenericPreprocessor):
+    """
+    3d_lowres and 3d_fullres are not resampled along z!
+    """
+    def resample_and_normalize(self, data, target_spacing, properties, seg=None, force_separate_z=None):
+        """
+        if target_spacing[0] is None or nan we use original_spacing_transposed[0] (no resampling along z)
+        :param data:
+        :param target_spacing:
+        :param properties:
+        :param seg:
+        :param force_separate_z:
+        :return:
+        """
+        original_spacing_transposed = np.array(properties["original_spacing"])[self.transpose_forward]
+        before = {
+            'spacing': properties["original_spacing"],
+            'spacing_transposed': original_spacing_transposed,
+            'data.shape (data is transposed)': data.shape
+        }
+
+        # remove nans
+        data[np.isnan(data)] = 0
+        target_spacing = deepcopy(target_spacing)
+        if target_spacing[0] is None or np.isnan(target_spacing[0]):
+            target_spacing[0] = original_spacing_transposed[0]
+        #print(target_spacing, original_spacing_transposed)
+        data, seg = resample_patient(data, seg, np.array(original_spacing_transposed), target_spacing, 3, 1,
+                                     force_separate_z=force_separate_z, order_z_data=0, order_z_seg=0,
+                                     separate_z_anisotropy_threshold=self.resample_separate_z_anisotropy_threshold)
+        after = {
+            'spacing': target_spacing,
+            'data.shape (data is resampled)': data.shape
+        }
+        st = "before:" + str(before) + '\nafter' + str(after) + "\n"
+        print(st)
+
+        if seg is not None:  # hippocampus 243 has one voxel with -2 as label. wtf?
+            seg[seg < -1] = 0
+
+        properties["size_after_resampling"] = data[0].shape
+        properties["spacing_after_resampling"] = target_spacing
+        use_nonzero_mask = self.use_nonzero_mask
+
+        assert len(self.normalization_scheme_per_modality) == len(data), "self.normalization_scheme_per_modality " \
+                                                                         "must have as many entries as data has " \
+                                                                         "modalities"
+        assert len(self.use_nonzero_mask) == len(data), "self.use_nonzero_mask must have as many entries as data" \
+                                                        " has modalities"
+
+        for c in range(len(data)):
+            scheme = self.normalization_scheme_per_modality[c]
+            if scheme == "CT":
+                # clip to lb and ub from train data foreground and use foreground mn and sd from training data
+                assert self.intensityproperties is not None, "ERROR: if there is a CT then we need intensity properties"
+                mean_intensity = self.intensityproperties[c]['mean']
+                std_intensity = self.intensityproperties[c]['sd']
+                lower_bound = self.intensityproperties[c]['percentile_00_5']
+                upper_bound = self.intensityproperties[c]['percentile_99_5']
+                data[c] = np.clip(data[c], lower_bound, upper_bound)
+                data[c] = (data[c] - mean_intensity) / std_intensity
+                if use_nonzero_mask[c]:
+                    data[c][seg[-1] < 0] = 0
+            elif scheme == "CT2":
+                # clip to lb and ub from train data foreground, use mn and sd form each case for normalization
+                assert self.intensityproperties is not None, "ERROR: if there is a CT then we need intensity properties"
+                lower_bound = self.intensityproperties[c]['percentile_00_5']
+                upper_bound = self.intensityproperties[c]['percentile_99_5']
+                mask = (data[c] > lower_bound) & (data[c] < upper_bound)
+                data[c] = np.clip(data[c], lower_bound, upper_bound)
+                mn = data[c][mask].mean()
+                sd = data[c][mask].std()
+                data[c] = (data[c] - mn) / sd
+                if use_nonzero_mask[c]:
+                    data[c][seg[-1] < 0] = 0
+            elif scheme == 'noNorm':
+                pass
+            else:
+                if use_nonzero_mask[c]:
+                    mask = seg[-1] >= 0
+                else:
+                    mask = np.ones(seg.shape[1:], dtype=bool)
+                data[c][mask] = (data[c][mask] - data[c][mask].mean()) / (data[c][mask].std() + 1e-8)
+                data[c][mask == 0] = 0
+        return data, seg, properties
+
+    def run(self, target_spacings, input_folder_with_cropped_npz, output_folder, data_identifier,
+            num_threads=default_num_threads, force_separate_z=None):
+        for i in range(len(target_spacings)):
+            target_spacings[i][0] = None
+        super().run(target_spacings, input_folder_with_cropped_npz, output_folder, data_identifier,
+                    default_num_threads, force_separate_z)
+
+
+class PreprocessorFor3D_NoResampling(GenericPreprocessor):
+    def resample_and_normalize(self, data, target_spacing, properties, seg=None, force_separate_z=None):
+        """
+        if target_spacing[0] is None or nan we use original_spacing_transposed[0] (no resampling along z)
+        :param data:
+        :param target_spacing:
+        :param properties:
+        :param seg:
+        :param force_separate_z:
+        :return:
+        """
+        original_spacing_transposed = np.array(properties["original_spacing"])[self.transpose_forward]
+        before = {
+            'spacing': properties["original_spacing"],
+            'spacing_transposed': original_spacing_transposed,
+            'data.shape (data is transposed)': data.shape
+        }
+
+        # remove nans
+        data[np.isnan(data)] = 0
+        target_spacing = deepcopy(original_spacing_transposed)
+        #print(target_spacing, original_spacing_transposed)
+        data, seg = resample_patient(data, seg, np.array(original_spacing_transposed), target_spacing, 3, 1,
+                                     force_separate_z=force_separate_z, order_z_data=0, order_z_seg=0,
+                                     separate_z_anisotropy_threshold=self.resample_separate_z_anisotropy_threshold)
+        after = {
+            'spacing': target_spacing,
+            'data.shape (data is resampled)': data.shape
+        }
+        st = "before:" + str(before) + '\nafter' + str(after) + "\n"
+        print(st)
+
+        if seg is not None:  # hippocampus 243 has one voxel with -2 as label. wtf?
+            seg[seg < -1] = 0
+
+        properties["size_after_resampling"] = data[0].shape
+        properties["spacing_after_resampling"] = target_spacing
+        use_nonzero_mask = self.use_nonzero_mask
+
+        assert len(self.normalization_scheme_per_modality) == len(data), "self.normalization_scheme_per_modality " \
+                                                                         "must have as many entries as data has " \
+                                                                         "modalities"
+        assert len(self.use_nonzero_mask) == len(data), "self.use_nonzero_mask must have as many entries as data" \
+                                                        " has modalities"
+
+        for c in range(len(data)):
+            scheme = self.normalization_scheme_per_modality[c]
+            if scheme == "CT":
+                # clip to lb and ub from train data foreground and use foreground mn and sd from training data
+                assert self.intensityproperties is not None, "ERROR: if there is a CT then we need intensity properties"
+                mean_intensity = self.intensityproperties[c]['mean']
+                std_intensity = self.intensityproperties[c]['sd']
+                lower_bound = self.intensityproperties[c]['percentile_00_5']
+                upper_bound = self.intensityproperties[c]['percentile_99_5']
+                data[c] = np.clip(data[c], lower_bound, upper_bound)
+                data[c] = (data[c] - mean_intensity) / std_intensity
+                if use_nonzero_mask[c]:
+                    data[c][seg[-1] < 0] = 0
+            elif scheme == "CT2":
+                # clip to lb and ub from train data foreground, use mn and sd form each case for normalization
+                assert self.intensityproperties is not None, "ERROR: if there is a CT then we need intensity properties"
+                lower_bound = self.intensityproperties[c]['percentile_00_5']
+                upper_bound = self.intensityproperties[c]['percentile_99_5']
+                mask = (data[c] > lower_bound) & (data[c] < upper_bound)
+                data[c] = np.clip(data[c], lower_bound, upper_bound)
+                mn = data[c][mask].mean()
+                sd = data[c][mask].std()
+                data[c] = (data[c] - mn) / sd
+                if use_nonzero_mask[c]:
+                    data[c][seg[-1] < 0] = 0
+            elif scheme == 'noNorm':
+                pass
+            else:
+                if use_nonzero_mask[c]:
+                    mask = seg[-1] >= 0
+                else:
+                    mask = np.ones(seg.shape[1:], dtype=bool)
+                data[c][mask] = (data[c][mask] - data[c][mask].mean()) / (data[c][mask].std() + 1e-8)
+                data[c][mask == 0] = 0
+        return data, seg, properties
+
+
 class PreprocessorFor2D_noNormalization(GenericPreprocessor):
     def resample_and_normalize(self, data, target_spacing, properties, seg=None, force_separate_z=None):
         original_spacing_transposed = np.array(properties["original_spacing"])[self.transpose_forward]
