@@ -13,10 +13,11 @@
 #    limitations under the License.
 
 from copy import deepcopy
+
+from batchgenerators.transforms.abstract_transforms import AbstractTransform
 from skimage.morphology import label, ball
 from skimage.morphology.binary import binary_erosion, binary_dilation, binary_closing, binary_opening
 import numpy as np
-from batchgenerators.transforms import AbstractTransform
 
 
 class RemoveRandomConnectedComponentFromOneHotEncodingTransform(AbstractTransform):
