@@ -13,11 +13,11 @@
 #    limitations under the License.
 
 from collections import OrderedDict
-from batchgenerators.augmentations.utils import random_crop_2D_image_batched, pad_nd_image
 import numpy as np
-from batchgenerators.dataloading import SlimDataLoaderBase
 from multiprocessing import Pool
 
+from batchgenerators.dataloading.data_loader import SlimDataLoaderBase
+
 from nnunet.configuration import default_num_threads
 from nnunet.paths import preprocessing_output_dir
 from batchgenerators.utilities.file_and_folder_operations import *
@@ -494,6 +494,7 @@ class DataLoader2D(SlimDataLoaderBase):
                     valid_slices = np.unique(voxels_of_that_class[:, 0])
                     random_slice = np.random.choice(valid_slices)
                     voxels_of_that_class = voxels_of_that_class[voxels_of_that_class[:, 0] == random_slice]
+                    voxels_of_that_class = voxels_of_that_class[:, 1:]
 
             # now crop case_all_data to contain just the slice of interest. If we want additional slice above and
             # below the current slice, here is where we get them. We stack those as additional color channels
